#include "dialog.h"
#include "ui_dialog.h"

#include <QProcess>
#include <QDirIterator>
#include <QFileDialog>
#include <QMessageBox>

QString oggenc_path;

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);
	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);

	ui->progressBar->setMinimum(0);
	ui->progressBar->setMaximum(1000);
	ui->progressBar->setValue(0);
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

class MyProcess : public QProcess {
public:
	qint64 readData(char *data, qint64 maxlen)
	{
		return QProcess::readData(data, maxlen);
	}
};

struct aborted {
};

bool Dialog::invoke_oggenc(QString infile, QString outfile)
{
	bool success = false;
	double perc = 0;
	QString text;
	MyProcess process;

	process.start("\"" + oggenc_path + "\" \"" + infile + "\" -o \"" + outfile + "\"");
//	process.start("\"" + oggenc_path + "\" -q 5 \"" + infile + "\" -o \"" + outfile + "\"");

	process.setReadChannel(QProcess::StandardError);

	while (1) {
		if (process.waitForFinished(100) || process.state() == QProcess::NotRunning) {
			filecount++;
			success = true;
			break;
		}
		if (interrupted) {
			process.kill();
			QCoreApplication::processEvents();
			process.terminate();
			QCoreApplication::processEvents();
			QFile file(outfile);
			file.remove();
			throw aborted();
		}
		while (process.waitForReadyRead(0)) {
			char tmp[1000];
			long long n = process.readData(tmp, sizeof(tmp));
			if (n < 1) {
				break;
			}
			text += QString::fromUtf8(tmp, (int)n);
			QCoreApplication::processEvents();
		}
		while (1) {
			int i = text.indexOf('\r');
			if (i == -1) {
				i = text.indexOf('\n');
				if (i == -1) {
					break;
				}
			}
			QString line = QString::fromUtf16(text.utf16(), i);
			i++;
			text = QString::fromUtf16(text.utf16() + i, text.size() - i);

			line = line.remove(' ');
			line = line.remove('\t');

			double d;
			if (swscanf((wchar_t const *)line.utf16(), L"[%lf%%]", &d) == 1) {
				perc = d;
			}
		}
		ui->progressBar->setValue((int)(perc * 10));
		QCoreApplication::processEvents();
	}

	if (success) {
		ui->progressBar->setValue(1000);
	} else {
		ui->progressBar->setValue(0);
	}
	QCoreApplication::processEvents();
	return success;
}

void Dialog::batch_convert(QString dir)
{
	QDirIterator it(dir);
	while (it.hasNext()) {
		if (interrupted) {
			break;
		}
		QString path = it.next();
		QFileInfo fi = it.fileInfo();
		if (!fi.fileName().startsWith('.')) {
			if (fi.isDir()) {
				batch_convert(path);
			} else {
				if (path.endsWith(".wav", Qt::CaseInsensitive)) {
					ui->label_dir->setText(dir);
					ui->label_name->setText(fi.baseName());
					QCoreApplication::processEvents();
					QString infile = path;
					QString outfile = QString::fromUtf16(path.utf16(), path.size() - 4) + ".ogg";
					QString orgfile = infile + '_';
					if (invoke_oggenc(infile, outfile)) {
						QFile::rename(infile, orgfile);
					}
					ui->label_name->clear();
				}
			}
		}
		QCoreApplication::processEvents();
	}
}

void Dialog::on_pushButton_Start_clicked()
{
	if (directory.isEmpty()) {
		QMessageBox::warning(this, this->windowTitle(), tr("Location not specified."));
		return;
	}
	filecount = 0;
	ui->pushButton_Start->setEnabled(false);
	ui->pushButton_Abort->setEnabled(true);
	try {
		interrupted = false;
		batch_convert(directory);
	} catch (aborted) {
	}
	ui->pushButton_Start->setEnabled(true);
	ui->pushButton_Abort->setEnabled(false);
	ui->progressBar->setValue(0);
	ui->label_dir->setText(directory);

	wchar_t tmp[1000];
	swprintf(tmp, (wchar_t const *)tr("%d files converted.").utf16(), filecount);
	QMessageBox::information(this, this->windowTitle(), QString::fromUtf16((ushort const *)tmp));
}


void Dialog::on_pushButton_Abort_clicked()
{
	interrupted = true;
	ui->pushButton_Start->setEnabled(true);
	ui->pushButton_Abort->setEnabled(false);
}

void Dialog::on_pushButton_Location_clicked()
{
	QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
	QString dir = QFileDialog::getExistingDirectory(this, tr("Select the folder."), directory, options);
	if (dir.isEmpty()) {
		return;
	}
	directory = dir;
	ui->label_dir->setText(directory);
}
